@@ -59,6 +59,7 @@ static struct sc_atr_table cardos_atrs[] = {
 	/* CardOS v5.0 */
 	{ "3b:d2:18:00:81:31:fe:58:c9:01:14", NULL, NULL, SC_CARD_TYPE_CARDOS_V5_0, 0, NULL},
 	/* CardOS v5.3 */
+	{ "3b:d2:18:00:81:31:fe:58:c9:02:17", NULL, NULL, SC_CARD_TYPE_CARDOS_V5_0, 0, NULL},
 	{ "3b:d2:18:00:81:31:fe:58:c9:03:16", NULL, NULL, SC_CARD_TYPE_CARDOS_V5_0, 0, NULL},
 	{ NULL, NULL, NULL, 0, 0, NULL }
 };
@@ -171,7 +172,7 @@ static int cardos_init(sc_card_t *card)
 	sc_apdu_t apdu;
 	u8 rbuf[2];
 
-	card->name = "CardOS M4";
+	card->name = "Atos CardOS";
 	card->cla = 0x00;
 
 	/* Set up algorithm info. */
@@ -298,7 +299,7 @@ static const struct sc_card_error cardos_errors[] = {
 /* no error, maybe a note */
 { 0x9000, SC_SUCCESS,		NULL}, 
 { 0x9001, SC_SUCCESS,		"success, but eeprom weakness detected"}, 
-{ 0x9850, SC_SUCCESS,		"over/underflow useing in/decrease"}
+{ 0x9850, SC_SUCCESS,		"over/underflow using in/decrease"}
 };
 
 static int cardos_check_sw(sc_card_t *card, unsigned int sw1, unsigned int sw2)
@@ -351,7 +352,7 @@ get_next_part:
 	len = apdu.resplen;
 	while (len != 0) {
 		size_t   tlen = 0, ilen = 0;
-		/* is there a file informatin block (0x6f) ? */
+		/* is there a file information block (0x6f) ? */
 		p = sc_asn1_find_tag(card->ctx, p, len, 0x6f, &tlen);
 		if (p == NULL) {
 			sc_debug(card->ctx, SC_LOG_DEBUG_NORMAL, "directory tag missing");
@@ -371,7 +372,7 @@ get_next_part:
 			buf[fids++] = q[1];
 			buflen -= 2;
 		} else
-			/* not enought space left in buffer => break */
+			/* not enough space left in buffer => break */
 			break;
 		/* extract next offset */
 		q = sc_asn1_find_tag(card->ctx, p, tlen, 0x8a, &ilen);
@@ -558,8 +559,8 @@ static int cardos_set_file_attributes(sc_card_t *card, sc_file_t *file)
 
 		status[0] = 0x01;
 		if (file->type == SC_FILE_TYPE_DF) {
-			status[1] = file->size >> 8;
-			status[2] = file->size;
+			status[1] = (file->size >> 8) & 0xFF;
+			status[2] = file->size & 0xFF;
 		} else {
 			status[1] = status[2] = 0x00; /* not used */
 		}
@@ -598,6 +599,8 @@ static int cardos_construct_fcp(sc_card_t *card, const sc_file_t *file,
 	/* we will add the length later  */
 	p++;
 
+	memset(buf, 0, sizeof(buf));
+
 	/* set the length */
 	buf[0] = (file->size >> 8) & 0xff;
 	buf[1] = file->size        & 0xff;
@@ -890,7 +893,7 @@ cardos_compute_signature(sc_card_t *card, const u8 *data, size_t datalen,
 	/* There are two ways to create a signature, depending on the way,
 	 * the key was created: RSA_SIG and RSA_PURE_SIG.
 	 * We can use the following reasoning, to determine the correct operation:
-	 * 1. We check for several caps flags (as set in card->caps), to pervent generating
+	 * 1. We check for several caps flags (as set in card->caps), to prevent generating
 	 *    invalid signatures with duplicated hash prefixes with some cards
 	 * 2. Use the information from AlgorithmInfo of the TokenInfo file.
 	 *    This information is parsed in set_security_env and stored in a static variable.
@@ -974,6 +977,32 @@ cardos_compute_signature(sc_card_t *card, const u8 *data, size_t datalen,
 	LOG_FUNC_RETURN(ctx, SC_ERROR_INTERNAL);
 }
 
+static int
+cardos_decipher(struct sc_card *card,
+		const u8 * crgram, size_t crgram_len,
+		u8 * out, size_t outlen)
+{
+	int r;
+	size_t card_max_send_size = card->max_send_size;
+	size_t reader_max_send_size = card->reader->max_send_size;
+
+	if (sc_get_max_send_size(card) < crgram_len + 1) {
+		/* CardOS doesn't support chaining for PSO:DEC, so we just _hope_
+		 * that both, the reader and the card are able to send enough data.
+		 * (data is prefixed with 1 byte padding content indicator) */
+		card->max_send_size = crgram_len + 1;
+		card->reader->max_send_size = crgram_len + 1;
+	}
+
+	r = iso_ops->decipher(card, crgram, crgram_len, out, outlen);
+
+	/* reset whatever we've modified above */
+	card->max_send_size = card_max_send_size;
+	card->reader->max_send_size = reader_max_send_size;
+
+	return r;
+}
+
 static int
 cardos_lifecycle_get(sc_card_t *card, int *mode)
 {
@@ -1277,6 +1306,7 @@ static struct sc_card_driver * sc_get_driver(void)
 	cardos_ops.set_security_env = cardos_set_security_env;
 	cardos_ops.restore_security_env = cardos_restore_security_env;
 	cardos_ops.compute_signature = cardos_compute_signature;
+	cardos_ops.decipher = cardos_decipher;
 
 	cardos_ops.list_files = cardos_list_files;
 	cardos_ops.check_sw = cardos_check_sw;
