@@ -241,7 +241,7 @@ static int muscle_update_binary(sc_card_t *card, unsigned int idx, const u8* buf
 		if(buffer == NULL) SC_FUNC_RETURN(card->ctx, SC_LOG_DEBUG_NORMAL, SC_ERROR_OUT_OF_MEMORY);
 
 		r = msc_read_object(card, objectId, 0, buffer, file->size);
-		/* TODO: RETREIVE ACLS */
+		/* TODO: RETRIEVE ACLS */
 		if(r < 0) goto update_bin_free_buffer;
 		r = msc_delete_object(card, objectId, 0);
 		if(r < 0) goto update_bin_free_buffer;
@@ -602,7 +602,7 @@ static int muscle_pin_cmd(sc_card_t *card, struct sc_pin_cmd_data *cmd,
 
 static int muscle_card_extract_key(sc_card_t *card, sc_cardctl_muscle_key_info_t *info)
 {
-	/* CURRENTLY DONT SUPPOT EXTRACTING PRIVATE KEYS... */
+	/* CURRENTLY DONT SUPPORT EXTRACTING PRIVATE KEYS... */
 	switch(info->keyType) {
 	case 1: /* RSA */
 		return msc_extract_rsa_public_key(card,
@@ -618,7 +618,7 @@ static int muscle_card_extract_key(sc_card_t *card, sc_cardctl_muscle_key_info_t
 
 static int muscle_card_import_key(sc_card_t *card, sc_cardctl_muscle_key_info_t *info)
 {
-	/* CURRENTLY DONT SUPPOT EXTRACTING PRIVATE KEYS... */
+	/* CURRENTLY DONT SUPPORT EXTRACTING PRIVATE KEYS... */
 	switch(info->keyType) {
 	case 0x02: /* RSA_PRIVATE */
 	case 0x03: /* RSA_PRIVATE_CRT */
@@ -721,7 +721,7 @@ static int muscle_decipher(sc_card_t * card,
 	u8 key_id;
 	int r;
 
-	/* saniti check */
+	/* sanity check */
 	if (priv->env.operation != SC_SEC_OPERATION_DECIPHER)
 		return SC_ERROR_INVALID_ARGUMENTS;
 
@@ -810,6 +810,21 @@ static int muscle_check_sw(sc_card_t * card, unsigned int sw1, unsigned int sw2)
 	return iso_ops->check_sw(card, sw1, sw2);
 }
 
+static int muscle_card_reader_lock_obtained(sc_card_t *card, int was_reset)
+{
+	int r = SC_SUCCESS;
+
+	SC_FUNC_CALLED(card->ctx, SC_LOG_DEBUG_VERBOSE);
+
+	if (was_reset > 0) {
+		if (msc_select_applet(card, muscleAppletId, sizeof muscleAppletId) != 1) {
+			r = SC_ERROR_INVALID_CARD;
+		}
+	}
+
+	LOG_FUNC_RETURN(card->ctx, r);
+}
+
 
 static struct sc_card_driver * sc_get_driver(void)
 {
@@ -837,6 +852,7 @@ static struct sc_card_driver * sc_get_driver(void)
 	muscle_ops.select_file = muscle_select_file;
 	muscle_ops.delete_file = muscle_delete_file;
 	muscle_ops.list_files = muscle_list_files;
+	muscle_ops.card_reader_lock_obtained = muscle_card_reader_lock_obtained;
 
 	return &muscle_drv;
 }
