@@ -40,10 +40,11 @@ static struct sc_atr_table tcos_atrs[] = {
 	{ "3B:BA:14:00:81:31:86:5D:00:64:05:14:02:02:31:80:90:00:91", NULL, NULL, SC_CARD_TYPE_TCOS_V2, 0, NULL },
 	/* Infineon SLE66CX320P */
 	{ "3B:BA:96:00:81:31:86:5D:00:64:05:60:02:03:31:80:90:00:66", NULL, NULL, SC_CARD_TYPE_TCOS_V2, 0, NULL },
-	/* Infoneon SLE66CX322P */
+	/* Infineon SLE66CX322P */
 	{ "3B:BA:96:00:81:31:86:5D:00:64:05:7B:02:03:31:80:90:00:7D", NULL, NULL, SC_CARD_TYPE_TCOS_V2, 0, NULL },
 	/* Philips P5CT072 */
 	{ "3B:BF:96:00:81:31:FE:5D:00:64:04:11:03:01:31:C0:73:F7:01:D0:00:90:00:7D", NULL, NULL, SC_CARD_TYPE_TCOS_V3, 0, NULL },
+	{ "3B:BF:96:00:81:31:FE:5D:00:64:04:11:04:0F:31:C0:73:F7:01:D0:00:90:00:74", NULL, NULL, SC_CARD_TYPE_TCOS_V3, 0, NULL },
 	/* Philips P5CT080 */
 	{ "3B:BF:B6:00:81:31:FE:5D:00:64:04:28:03:02:31:C0:73:F7:01:D0:00:90:00:67", NULL, NULL, SC_CARD_TYPE_TCOS_V3, 0, NULL },
 	{ NULL, NULL, NULL, 0, 0, NULL }
@@ -545,7 +546,7 @@ static int tcos_set_security_env(sc_card_t *card, const sc_security_env_t *env,
 	p = sbuf;
 	*p++=0x80; *p++=0x01; *p++=tcos3 ? 0x0A : 0x10;
 	if (env->flags & SC_SEC_ENV_KEY_REF_PRESENT) {
-		*p++ = (env->flags & SC_SEC_ENV_KEY_REF_ASYMMETRIC) ? 0x83 : 0x84;
+		*p++ = (env->flags & SC_SEC_ENV_KEY_REF_SYMMETRIC) ? 0x83 : 0x84;
 		*p++ = env->key_ref_len;
 		memcpy(p, env->key_ref, env->key_ref_len);
 		p += env->key_ref_len;
@@ -705,33 +706,25 @@ static int tcos_setperm(sc_card_t *card, int enable_nullpin)
 
 static int tcos_get_serialnr(sc_card_t *card, sc_serial_number_t *serial)
 {
-	int       r;
-	u8        buf[64];
-	size_t    len;
-	sc_path_t tpath;
-	sc_file_t *tfile = NULL;
+	int r;
 
-	if (!serial) return SC_ERROR_INVALID_ARGUMENTS;
+	if (!serial)
+		return SC_ERROR_INVALID_ARGUMENTS;
 
 	/* see if we have cached serial number */
 	if (card->serialnr.len) {
 		memcpy(serial, &card->serialnr, sizeof(*serial));
 		return SC_SUCCESS;
 	}
-	sc_format_path("3F002F02", &tpath);
-	r = sc_select_file(card, &tpath, &tfile);
-	if (r < 0) return r;
 
-	len = tfile->size;
-	sc_file_free(tfile);
-	if (len > sizeof(buf) || len < 12) return SC_ERROR_INTERNAL;
+	card->serialnr.len = sizeof card->serialnr.value;
+	r = sc_parse_ef_gdo(card, card->serialnr.value, &card->serialnr.len, NULL, 0);
+	if (r < 0) {
+		card->serialnr.len = 0;
+		return r;
+	}
 
-	r = sc_read_binary(card, 0, buf, len, 0);
-	if (r < 0) return r;
-	if (buf[0] != 0x5a || buf[1] > len - 2) return SC_ERROR_INTERNAL;
-
-	card->serialnr.len = buf[1];	
-	memcpy(card->serialnr.value, buf+2, buf[1]);
+	/* copy and return serial number */
 	memcpy(serial, &card->serialnr, sizeof(*serial));
 
 	return SC_SUCCESS;
