@@ -191,7 +191,7 @@ static int read_file(sc_pkcs15_card_t * p15card, u8 fid[2],
 	path.aid = sc_hsm_aid;
 	/* we don't have a pre-known size of the file */
 	path.count = -1;
-	if (!p15card->opts.use_file_cache
+	if (!p15card->opts.use_file_cache || !efbin
 			|| SC_SUCCESS != sc_pkcs15_read_cached_file(p15card, &path, &efbin, len)) {
 		/* avoid re-selection of SC-HSM */
 		path.aid.len = 0;
@@ -606,13 +606,13 @@ static int sc_pkcs15emu_sc_hsm_add_prkd(sc_pkcs15_card_t * p15card, u8 keyid) {
 	/* Try to select a related EF containing the PKCS#15 description of the key */
 	len = sizeof efbin;
 	r = read_file(p15card, fid, efbin, &len, 1);
-	LOG_TEST_RET(card->ctx, r, "Could not read EF.PRKD");
+	LOG_TEST_RET(card->ctx, r, "Skipping optional EF.PRKD");
 
 	ptr = efbin;
 
 	memset(&prkd, 0, sizeof(prkd));
 	r = sc_pkcs15_decode_prkdf_entry(p15card, &prkd, (const u8 **)&ptr, &len);
-	LOG_TEST_RET(card->ctx, r, "Could not decode EF.PRKD");
+	LOG_TEST_RET(card->ctx, r, "Skipping optional EF.PRKD");
 
 	/* All keys require user PIN authentication */
 	prkd.auth_id.len = 1;
@@ -642,8 +642,6 @@ static int sc_pkcs15emu_sc_hsm_add_prkd(sc_pkcs15_card_t * p15card, u8 keyid) {
 	r = read_file(p15card, fid, efbin, &len, 0);
 	LOG_TEST_RET(card->ctx, r, "Could not read EF");
 
-	LOG_TEST_RET(card->ctx, r, "Could not read EF");
-
 	if (efbin[0] == 0x67) {		/* Decode CSR and create public key object */
 		sc_pkcs15emu_sc_hsm_add_pubkey(p15card, efbin, len, key_info, prkd.label);
 		free(key_info);
@@ -702,13 +700,13 @@ static int sc_pkcs15emu_sc_hsm_add_dcod(sc_pkcs15_card_t * p15card, u8 id) {
 	/* Try to select a related EF containing the PKCS#15 description of the data */
 	len = sizeof efbin;
 	r = read_file(p15card, fid, efbin, &len, 1);
-	LOG_TEST_RET(card->ctx, r, "Could not read EF.DCOD");
+	LOG_TEST_RET(card->ctx, r, "Skipping optional EF.DCOD");
 
 	ptr = efbin;
 
 	memset(&data_obj, 0, sizeof(data_obj));
 	r = sc_pkcs15_decode_dodf_entry(p15card, &data_obj, &ptr, &len);
-	LOG_TEST_RET(card->ctx, r, "Could not decode EF.DCOD");
+	LOG_TEST_RET(card->ctx, r, "Could not decode optional EF.DCOD");
 
 	data_info = (sc_pkcs15_data_info_t *)data_obj.data;
 
@@ -741,13 +739,13 @@ static int sc_pkcs15emu_sc_hsm_add_cd(sc_pkcs15_card_t * p15card, u8 id) {
 	/* Try to select a related EF containing the PKCS#15 description of the data */
 	len = sizeof efbin;
 	r = read_file(p15card, fid, efbin, &len, 1);
-	LOG_TEST_RET(card->ctx, r, "Could not read EF.DCOD");
+	LOG_TEST_RET(card->ctx, r, "Skipping optional EF.DCOD");
 
 	ptr = efbin;
 
 	memset(&obj, 0, sizeof(obj));
 	r = sc_pkcs15_decode_cdf_entry(p15card, &obj, &ptr, &len);
-	LOG_TEST_RET(card->ctx, r, "Could not decode EF.CD");
+	LOG_TEST_RET(card->ctx, r, "Skipping optional EF.CDOD");
 
 	cert_info = (sc_pkcs15_cert_info_t *)obj.data;
 
@@ -772,10 +770,10 @@ static int sc_pkcs15emu_sc_hsm_read_tokeninfo (sc_pkcs15_card_t * p15card)
 	/* Read token info */
 	len = sizeof efbin;
 	r = read_file(p15card, (u8 *) "\x2F\x03", efbin, &len, 1);
-	LOG_TEST_RET(card->ctx, r, "Could not read EF.TokenInfo");
+	LOG_TEST_RET(card->ctx, r, "Skipping optional EF.TokenInfo");
 
 	r = sc_pkcs15_parse_tokeninfo(card->ctx, p15card->tokeninfo, efbin, len);
-	LOG_TEST_RET(card->ctx, r, "Could not decode EF.TokenInfo");
+	LOG_TEST_RET(card->ctx, r, "Skipping optional EF.TokenInfo");
 
 	LOG_FUNC_RETURN(card->ctx, SC_SUCCESS);
 }
@@ -823,8 +821,10 @@ static int sc_pkcs15emu_sc_hsm_init (sc_pkcs15_card_t * p15card)
 
 	p15card->card->version.hw_major = 24;	/* JCOP 2.4.1r3 */
 	p15card->card->version.hw_minor = 13;
-	p15card->card->version.fw_major = file->prop_attr[file->prop_attr_len - 2];
-	p15card->card->version.fw_minor = file->prop_attr[file->prop_attr_len - 1];
+	if (file && file->prop_attr && file->prop_attr_len >= 2) {
+		p15card->card->version.fw_major = file->prop_attr[file->prop_attr_len - 2];
+		p15card->card->version.fw_minor = file->prop_attr[file->prop_attr_len - 1];
+	}
 
 	sc_file_free(file);
 
@@ -835,7 +835,7 @@ static int sc_pkcs15emu_sc_hsm_init (sc_pkcs15_card_t * p15card)
 	} else {
 		len = sizeof efbin;
 		r = read_file(p15card, (u8 *) "\x2F\x02", efbin, &len, 1);
-		LOG_TEST_RET(card->ctx, r, "Could not select EF.C_DevAut");
+		LOG_TEST_RET(card->ctx, r, "Skipping optional EF.C_DevAut");
 
 		/* save EF_C_DevAut for further use */
 		ptr = realloc(priv->EF_C_DevAut, len);
@@ -855,7 +855,12 @@ static int sc_pkcs15emu_sc_hsm_init (sc_pkcs15_card_t * p15card)
 	sc_pkcs15emu_sc_hsm_read_tokeninfo(p15card);
 
 	if (p15card->tokeninfo->label == NULL) {
-		p15card->tokeninfo->label = strdup("SmartCard-HSM");
+		if (p15card->card->type == SC_CARD_TYPE_SC_HSM_GOID
+				|| p15card->card->type == SC_CARD_TYPE_SC_HSM_SOC) {
+			p15card->tokeninfo->label = strdup("GoID");
+		} else {
+			p15card->tokeninfo->label = strdup("SmartCard-HSM");
+		}
 		if (p15card->tokeninfo->label == NULL)
 			LOG_FUNC_RETURN(card->ctx, SC_ERROR_OUT_OF_MEMORY);
 	}
@@ -866,7 +871,12 @@ static int sc_pkcs15emu_sc_hsm_init (sc_pkcs15_card_t * p15card)
 	}
 
 	if (p15card->tokeninfo->manufacturer_id == NULL) {
-		p15card->tokeninfo->manufacturer_id = strdup("www.CardContact.de");
+		if (p15card->card->type == SC_CARD_TYPE_SC_HSM_GOID
+				|| p15card->card->type == SC_CARD_TYPE_SC_HSM_SOC) {
+			p15card->tokeninfo->manufacturer_id = strdup("Bundesdruckerei GmbH");
+		} else {
+			p15card->tokeninfo->manufacturer_id = strdup("www.CardContact.de");
+		}
 		if (p15card->tokeninfo->manufacturer_id == NULL)
 			LOG_FUNC_RETURN(card->ctx, SC_ERROR_OUT_OF_MEMORY);
 	}
@@ -941,7 +951,8 @@ static int sc_pkcs15emu_sc_hsm_init (sc_pkcs15_card_t * p15card)
 		LOG_FUNC_RETURN(card->ctx, r);
 
 
-	if (card->type == SC_CARD_TYPE_SC_HSM_SOC) {
+	if (card->type == SC_CARD_TYPE_SC_HSM_SOC
+			|| card->type == SC_CARD_TYPE_SC_HSM_GOID) {
 		/* SC-HSM of this type always has a PIN-Pad */
 		r = SC_SUCCESS;
 	} else {
@@ -998,7 +1009,8 @@ int sc_pkcs15emu_sc_hsm_init_ex(sc_pkcs15_card_t *p15card,
 		return sc_pkcs15emu_sc_hsm_init(p15card);
 	} else {
 		if (p15card->card->type != SC_CARD_TYPE_SC_HSM
-				&& p15card->card->type != SC_CARD_TYPE_SC_HSM_SOC) {
+				&& p15card->card->type != SC_CARD_TYPE_SC_HSM_SOC
+				&& p15card->card->type != SC_CARD_TYPE_SC_HSM_GOID) {
 			return SC_ERROR_WRONG_CARD;
 		}
 		return sc_pkcs15emu_sc_hsm_init(p15card);
