@@ -762,7 +762,7 @@ static int pcsc_init(sc_context_t *ctx)
 	/* PC/SC Defaults */
 	gpriv->provider_library = DEFAULT_PCSC_PROVIDER;
 	gpriv->connect_exclusive = 0;
-	gpriv->disconnect_action = SCARD_RESET_CARD;
+	gpriv->disconnect_action = SCARD_LEAVE_CARD;
 	gpriv->transaction_end_action = SCARD_LEAVE_CARD;
 	gpriv->reconnect_action = SCARD_LEAVE_CARD;
 	gpriv->enable_pinpad = 1;
@@ -781,7 +781,7 @@ static int pcsc_init(sc_context_t *ctx)
 		gpriv->connect_exclusive =
 			scconf_get_bool(conf_block, "connect_exclusive", gpriv->connect_exclusive);
 		gpriv->disconnect_action =
-			pcsc_reset_action(scconf_get_str(conf_block, "disconnect_action", "reset"));
+			pcsc_reset_action(scconf_get_str(conf_block, "disconnect_action", "leave"));
 		gpriv->transaction_end_action =
 			pcsc_reset_action(scconf_get_str(conf_block, "transaction_end_action", "leave"));
 		gpriv->reconnect_action =
@@ -1219,11 +1219,10 @@ static void detect_reader_features(sc_reader_t *reader, SCARDHANDLE card_handle)
 			reader->vendor = strdup((char *) rbuf);
 		}
 
-		rcount = sizeof rbuf;
+		rcount = sizeof i;
 		if(gpriv->SCardGetAttrib(card_handle, SCARD_ATTR_VENDOR_IFD_VERSION,
-					rbuf, &rcount) == SCARD_S_SUCCESS
-				&& rcount == 4) {
-			i = *(DWORD *) rbuf;
+					(u8 *) &i, &rcount) == SCARD_S_SUCCESS
+				&& rcount == sizeof i) {
 			reader->version_major = (i >> 24) & 0xFF;
 			reader->version_minor = (i >> 16) & 0xFF;
 		}
@@ -1400,7 +1399,7 @@ static int pcsc_detect_readers(sc_context_t *ctx)
 		}
 		if (rv == (LONG)SCARD_E_SHARING_VIOLATION) {
 			/* Assume that there is a card in the reader in shared mode if
-			 * direct communcation failed */
+			 * direct communication failed */
 			rv = gpriv->SCardConnect(gpriv->pcsc_ctx, reader->name,
 					SCARD_SHARE_SHARED,
 					SCARD_PROTOCOL_T0|SCARD_PROTOCOL_T1, &card_handle,
@@ -1459,7 +1458,8 @@ static int pcsc_wait_for_event(sc_context_t *ctx, unsigned int event_mask, sc_re
 			rgReaderStates[i].dwCurrentState = SCARD_STATE_UNAWARE;
 			rgReaderStates[i].dwEventState = SCARD_STATE_UNAWARE;
 		}
-#ifndef __APPLE__ /* OS X 10.6.2 does not support PnP notification */
+#ifndef __APPLE__
+	   	/* OS X 10.6.2 - 10.12.6 do not support PnP notification */
 		if (event_mask & SC_EVENT_READER_ATTACHED) {
 			rgReaderStates[i].szReader = "\\\\?PnP?\\Notification";
 			rgReaderStates[i].dwCurrentState = SCARD_STATE_UNAWARE;
@@ -1492,12 +1492,14 @@ static int pcsc_wait_for_event(sc_context_t *ctx, unsigned int event_mask, sc_re
 		goto out;
 	}
 
+#ifdef __APPLE__
 	if (num_watch == 0) {
 		sc_log(ctx, "No readers available, PnP notification not supported");
 		*event_reader = NULL;
 		r = SC_ERROR_NO_READERS_FOUND;
 		goto out;
 	}
+#endif
 
 	rv = gpriv->SCardGetStatusChange(gpriv->pcsc_wait_ctx, 0, rgReaderStates, num_watch);
 	if (rv != SCARD_S_SUCCESS) {
@@ -1703,17 +1705,12 @@ static int part10_build_verify_pin_block(struct sc_reader *reader, u8 * buf, siz
 	pin_verify->bTeoPrologue[2] = 0x00;
 
 	/* APDU itself */
-	pin_verify->abData[offset++] = apdu->cla;
-	pin_verify->abData[offset++] = apdu->ins;
-	pin_verify->abData[offset++] = apdu->p1;
-	pin_verify->abData[offset++] = apdu->p2;
-
-	/* Copy data if not Case 1 */
-	if (data->pin1.length_offset != 4) {
-		pin_verify->abData[offset++] = apdu->lc;
-		memcpy(&pin_verify->abData[offset], apdu->data, apdu->datalen);
-		offset += apdu->datalen;
-	}
+	LOG_TEST_RET(reader->ctx,
+			sc_apdu2bytes(reader->ctx, apdu,
+				reader->active_protocol, pin_verify->abData,
+				SC_MAX_APDU_BUFFER_SIZE),
+			"Could not encode PIN APDU");
+	offset += sc_apdu_get_length(apdu, reader->active_protocol);
 
 	pin_verify->ulDataLength = HOST_TO_CCID_32(offset); /* APDU size */
 
@@ -1822,17 +1819,12 @@ static int part10_build_modify_pin_block(struct sc_reader *reader, u8 * buf, siz
 	pin_modify->bTeoPrologue[2] = 0x00;
 
 	/* APDU itself */
-	pin_modify->abData[offset++] = apdu->cla;
-	pin_modify->abData[offset++] = apdu->ins;
-	pin_modify->abData[offset++] = apdu->p1;
-	pin_modify->abData[offset++] = apdu->p2;
-
-	/* Copy data if not Case 1 */
-	if (pin_ref->length_offset != 4) {
-		pin_modify->abData[offset++] = apdu->lc;
-		memcpy(&pin_modify->abData[offset], apdu->data, apdu->datalen);
-		offset += apdu->datalen;
-	}
+	LOG_TEST_RET(reader->ctx,
+			sc_apdu2bytes(reader->ctx, apdu,
+				reader->active_protocol, pin_modify->abData,
+				SC_MAX_APDU_BUFFER_SIZE),
+			"Could not encode PIN APDU");
+	offset += sc_apdu_get_length(apdu, reader->active_protocol);
 
 	pin_modify->ulDataLength = HOST_TO_CCID_32(offset); /* APDU size */
 
