@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2010-2012 Frank Morgner <frankmorgner@gmail.com>
+ * Copyright (C) 2010-2018 Frank Morgner <frankmorgner@gmail.com>
  *
  * This file is part of OpenSC.
  *
@@ -30,6 +30,7 @@
 #include <libopensc/log.h>
 #include <libopensc/opensc.h>
 #include <libopensc/sm.h>
+#include <libopensc/card-npa.h>
 #include <sm/sm-eac.h>
 #include <stdint.h>
 #include <stdio.h>
@@ -172,7 +173,7 @@ static void read_dg(sc_card_t *card, unsigned char sfid, const char *dg_str,
 				sfid, dg_str, sc_strerror(r));
 	else {
 		char buf[0x200];
-		sc_hex_dump(NULL, 0, *dg, *dg_len, buf, sizeof buf);
+		sc_hex_dump(*dg, *dg_len, buf, sizeof buf);
 		fprintf(stdout, "Read %s", buf);
 	}
 }
@@ -319,7 +320,7 @@ static int add_to_ASN1_AUXILIARY_DATA(
 	if (data && data_len) {
 		template->discretionary_data3 = ASN1_OCTET_STRING_new();
 		if (!template->discretionary_data3
-				|| !M_ASN1_OCTET_STRING_set(
+				|| !ASN1_OCTET_STRING_set(
 					template->discretionary_data3, data, data_len)) {
 			r = SC_ERROR_INTERNAL;
 			goto err;
@@ -359,7 +360,7 @@ main (int argc, char **argv)
 
 	sc_context_t *ctx = NULL;
 	sc_card_t *card = NULL;
-	sc_reader_t *reader;
+	sc_reader_t *reader = NULL;
 
 	int r, tr_version = EAC_TR_VERSION_2_02;
 	struct establish_pace_channel_input pace_input;
@@ -390,12 +391,13 @@ main (int argc, char **argv)
 		pin = getenv("PIN");
 		puk = getenv("PUK");
 		newpin = getenv("NEWPIN");
+	} else {
+		can = cmdline.can_arg;
+		mrz = cmdline.mrz_arg;
+		pin = cmdline.pin_arg;
+		puk = cmdline.puk_arg;
+		newpin = cmdline.new_pin_arg;
 	}
-	can = cmdline.can_arg;
-	mrz = cmdline.mrz_arg;
-	pin = cmdline.pin_arg;
-	puk = cmdline.puk_arg;
-	newpin = cmdline.new_pin_arg;
 	if (cmdline.chat_given) {
 		pace_input.chat = chat;
 		pace_input.chat_length = sizeof chat;
@@ -418,11 +420,11 @@ main (int argc, char **argv)
 	if (cmdline.tr_03110v201_flag)
 		tr_version = EAC_TR_VERSION_2_01;
 	if (cmdline.disable_all_checks_flag)
-		npa_default_flags |= NPA_FLAG_DISABLE_CHECK_ALL;
+		eac_default_flags |= EAC_FLAG_DISABLE_CHECK_ALL;
 	if (cmdline.disable_ta_checks_flag)
-		npa_default_flags |= NPA_FLAG_DISABLE_CHECK_TA;
+		eac_default_flags |= EAC_FLAG_DISABLE_CHECK_TA;
 	if (cmdline.disable_ca_checks_flag)
-		npa_default_flags |= NPA_FLAG_DISABLE_CHECK_CA;
+		eac_default_flags |= EAC_FLAG_DISABLE_CHECK_CA;
 
 
 	r = initialize(cmdline.reader_arg, cmdline.verbose_given, &ctx, &reader);
@@ -456,12 +458,12 @@ main (int argc, char **argv)
 			if (pin) {
 				if (sscanf(pin, "%llu", &secret) != 1) {
 					fprintf(stderr, "%s is not an unsigned long long.\n",
-							npa_secret_name(pace_input.pin_id));
+							eac_secret_name(pace_input.pin_id));
 					exit(2);
 				}
 				if (strlen(pin) > pace_input.pin_length) {
 					fprintf(stderr, "%s too big, only %u digits allowed.\n",
-							npa_secret_name(pace_input.pin_id),
+							eac_secret_name(pace_input.pin_id),
 							(unsigned int) pace_input.pin_length);
 					exit(2);
 				}
@@ -473,12 +475,12 @@ main (int argc, char **argv)
 			if (can) {
 				if (sscanf(can, "%llu", &secret) != 1) {
 					fprintf(stderr, "%s is not an unsigned long long.\n",
-							npa_secret_name(pace_input.pin_id));
+							eac_secret_name(pace_input.pin_id));
 					exit(2);
 				}
 				if (strlen(can) > pace_input.pin_length) {
 					fprintf(stderr, "%s too big, only %u digits allowed.\n",
-							npa_secret_name(pace_input.pin_id),
+							eac_secret_name(pace_input.pin_id),
 							(unsigned int) pace_input.pin_length);
 					exit(2);
 				}
@@ -490,12 +492,12 @@ main (int argc, char **argv)
 			if (puk) {
 				if (sscanf(puk, "%llu", &secret) != 1) {
 					fprintf(stderr, "%s is not an unsigned long long.\n",
-							npa_secret_name(pace_input.pin_id));
+							eac_secret_name(pace_input.pin_id));
 					exit(2);
 				}
 				if (strlen(puk) > pace_input.pin_length) {
 					fprintf(stderr, "%s too big, only %u digits allowed.\n",
-							npa_secret_name(pace_input.pin_id),
+							eac_secret_name(pace_input.pin_id),
 							(unsigned int) pace_input.pin_length);
 					exit(2);
 				}
@@ -514,7 +516,7 @@ main (int argc, char **argv)
 			gettimeofday(&tv, NULL);
 			printf("%u,%06u: Trying %s=%s\n",
 					(unsigned int) tv.tv_sec, (unsigned int) tv.tv_usec,
-					npa_secret_name(pace_input.pin_id), pace_input.pin);
+					eac_secret_name(pace_input.pin_id), pace_input.pin);
 
 			r = perform_pace(card, pace_input, &pace_output, tr_version);
 
@@ -525,12 +527,12 @@ main (int argc, char **argv)
 		if (0 > r) {
 			printf("%u,%06u: Tried breaking %s without success.\n",
 					(unsigned int) tv.tv_sec, (unsigned int) tv.tv_usec,
-					npa_secret_name(pace_input.pin_id));
+					eac_secret_name(pace_input.pin_id));
 			goto err;
 		} else {
 			printf("%u,%06u: Tried breaking %s with success (=%s).\n",
 					(unsigned int) tv.tv_sec, (unsigned int) tv.tv_usec,
-					npa_secret_name(pace_input.pin_id),
+					eac_secret_name(pace_input.pin_id),
 					pace_input.pin);
 		}
 	}
@@ -749,7 +751,7 @@ main (int argc, char **argv)
 		if (r < 0)
 			goto err;
 		printf("Established PACE channel with %s.\n",
-				npa_secret_name(pace_input.pin_id));
+				eac_secret_name(pace_input.pin_id));
 
 nopace:
 		if (cmdline.cv_certificate_given || cmdline.private_key_given) {
