@@ -449,6 +449,10 @@ int do_genkey(sc_card_t *card, u8 key_id, unsigned int key_len)
 	}
 	sc_format_path("006E007300C5", &path);
 	r = sc_select_file(card, &path, &file);
+	if (r < 0) {
+		printf("Failed to retrieve fingerprints. Error %s.\n", sc_strerror(r));
+		return 1;
+	}
 	r = sc_read_binary(card, 0, fingerprints, 60, 0);
 	if (r < 0) {
 		printf("Failed to retrieve fingerprints. Error %s.\n", sc_strerror(r));
@@ -527,7 +531,7 @@ int delete_key_openpgp(sc_card_t *card, u8 key_id)
 		/* Send APDU to card */
 		r = sc_transmit_apdu(card, &apdu);
 		if (r) {
-			fprintf(stderr, "Transmiting APDU failed: %s\n", sc_strerror(r));
+			fprintf(stderr, "Transmitting APDU failed: %s\n", sc_strerror(r));
 			return r;
 		}
 	}
@@ -568,14 +572,8 @@ int do_delete_key(sc_card_t *card, u8 key_id)
 
 int do_erase(sc_card_t *card)
 {
-	int r;
-	/* Check card version */
-	if (card->type != SC_CARD_TYPE_OPENPGP_V2) {
-		printf("Do not erase card which is not OpenPGP v2\n");
-	}
 	printf("Erase card\n");
-	r = sc_card_ctl(card, SC_CARDCTL_ERASE_CARD, NULL);
-	return r;
+	return sc_card_ctl(card, SC_CARDCTL_ERASE_CARD, NULL);
 }
 
 int main(int argc, char **argv)
@@ -615,9 +613,11 @@ int main(int argc, char **argv)
 	}
 
 	/* check card type */
-	if ((card->type != SC_CARD_TYPE_OPENPGP_V1) &&
-		(card->type != SC_CARD_TYPE_OPENPGP_V2) &&
-		(card->type != SC_CARD_TYPE_OPENPGP_GNUK)) {
+	if ((card->type != SC_CARD_TYPE_OPENPGP_BASE) &&
+			(card->type != SC_CARD_TYPE_OPENPGP_V1) &&
+			(card->type != SC_CARD_TYPE_OPENPGP_V2) &&
+			(card->type != SC_CARD_TYPE_OPENPGP_V3) &&
+			(card->type != SC_CARD_TYPE_OPENPGP_GNUK)) {
 		util_error("not an OpenPGP card");
 		fprintf(stderr, "Card type %X\n", card->type);
 		exit_status = EXIT_FAILURE;
