@@ -498,8 +498,8 @@ static int list_files(void)
 static int send_apdu(void)
 {
 	sc_apdu_t apdu;
-	u8 buf[SC_MAX_APDU_BUFFER_SIZE],
-	  rbuf[SC_MAX_APDU_BUFFER_SIZE];
+	u8 buf[SC_MAX_EXT_APDU_BUFFER_SIZE],
+	  rbuf[SC_MAX_EXT_APDU_BUFFER_SIZE];
 	size_t len0, r;
 	int c;
 
@@ -668,7 +668,7 @@ static int card_reset(const char *reset_type)
 	return 0;
 }
 
-int main(int argc, char * const argv[])
+int main(int argc, char *argv[])
 {
 	int err = 0, r, c, long_optind = 0;
 	int do_info = 0;
