@@ -30,6 +30,7 @@
 #endif
 #include <ctype.h>
 #include "util.h"
+#include "ui/notify.h"
 
 int
 is_string_valid_atr(const char *atr_str)
@@ -54,6 +55,8 @@ util_connect_card_ex(sc_context_t *ctx, sc_card_t **cardp,
 	struct sc_card *card = NULL;
 	int r;
 
+	sc_notify_init();
+
 	if (do_wait) {
 		unsigned int event;
 
@@ -232,7 +235,8 @@ void util_hex_dump_asc(FILE *f, const u8 *in, size_t count, int addr)
 	}
 }
 
-void util_print_usage_and_die(const char *app_name, const struct option options[],
+NORETURN void
+util_print_usage_and_die(const char *app_name, const struct option options[],
 	const char *option_help[], const char *args)
 {
 	int i;
@@ -343,7 +347,7 @@ const char * util_acl_to_str(const sc_acl_entry_t *e)
 	return line;
 }
 
-void
+NORETURN void
 util_fatal(const char *fmt, ...)
 {
 	va_list	ap;
@@ -353,6 +357,9 @@ util_fatal(const char *fmt, ...)
 	vfprintf(stderr, fmt, ap);
 	fprintf(stderr, "\nAborting.\n");
 	va_end(ap);
+
+	sc_notify_close();
+
 	exit(1);
 }
 
